// Copyright (c) 2022-2023 VMware Carbon Black, all rights reserved

// Package deploymenttype -- enumeration copied from the monolith concerned with deployment types
package deploymenttype

import (
	"strings"
)

// DeploymentType -- Type alias of allowed deployment types
type DeploymentType string

// IsKnown -- Returns true if a specified deployment type is a known definition
func (t DeploymentType) IsKnown() bool {
	for _, element := range AllDeploymentTypes {
		if t == element {
			return true
		}
	}
	return false
}

// NOTE: Changes here should also be reflected in `test/functional/suites/lib/models/enumerations.py`

// NOTE: Keep in sync with:
//   - https://gitlab.bit9.local/cb-defense/cbd/-/blob/main/backend/domainModel/src/main/java/com/scargo/domainmodel/entity/support/DeploymentType.java
const (
	UNKNOWN  DeploymentType = "UNKNOWN"  // An unknown deployment type
	ENDPOINT DeploymentType = "ENDPOINT" // A generic device on which a sensor is installed. It is usually installed on an end-user's device, and is managed either manually or through an imaged OS
	WORKLOAD DeploymentType = "WORKLOAD" // A device whose sensor is managed by Cloud Workload Protection's Lifecycle Management service
	VDI      DeploymentType = "VDI"      // A device whose sensor is running on a cloned VDI which is managed by providers like Horizon, Citrix
	AWS      DeploymentType = "AWS"      // A device whose sensor is running on an AWS EC2 instance deployed on AWS EC2 service
	AZURE    DeploymentType = "AZURE"    // A device whose sensor is running on an AZURE instance
	GCP      DeploymentType = "GCP"      // A device is a device whose sensor is running on a GCP instance
)

// AllDeploymentTypes -- List of all accepted DeploymentType
var AllDeploymentTypes = []DeploymentType{ //nolint:gochecknoglobals  // this is our enum
	ENDPOINT,
	WORKLOAD,
	VDI,
	AWS,
	AZURE,
	GCP,
}

// FromString -- Convert incoming string to a DeploymentType
//   - will be UNKNOWN if empty string or all spaces
func FromString(deploymentType string) DeploymentType {
	result := UNKNOWN

	if strings.TrimSpace(deploymentType) != "" {
		result = DeploymentType(strings.TrimSpace(strings.ToUpper(deploymentType)))
	}
	return result
}
